/****************************************************************************
Copyright 2015-2020, Enrique Sergio Burgos

This file is part of GUI Editor for Octave.

GUI Editor for Octave is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

GUI Editor is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GUI Editor.  If not, see <http://www.gnu.org/licenses/>.
****************************************************************************/
#include "commonproperties.h"
#include <QRegExp>
#include <QWidget>
#include <QFontDatabase>
#include "mainwnd.h"
#include "wdgclrchooser.h"

positionPropEditor::positionPropEditor(abstractUICCtrl *ctrl):abstractPropEditor(ctrl)
{
    this->propName = "Position";
}

QString positionPropEditor::getValue()
{
    QString ret;
    ret = "[";
    ret += QString::number(this->ctrl->associatedWidget()->x()) + " ";
    ret += QString::number(this->ctrl->associatedWidget()->y()) + " ";
    ret += QString::number(this->ctrl->associatedWidget()->width()) + " ";
    ret += QString::number(this->ctrl->associatedWidget()->height()) + "]";
    return ret;
}

bool positionPropEditor::isValidValue(QString newVal)
{
    QRegExp r("^\\[[-+]?\\d{1,10}\\s[-+]?\\d{1,10}\\s\\d{1,10}\\s\\d{1,10}\\]$");
    return newVal.contains(r) == 1;
}

void positionPropEditor::setValue(QString newVal)
{
    QStringList v;
    int w, h;
    if(this->isValidValue(newVal))
    {
        newVal.remove("[");
        newVal.remove("]");
        v = newVal.split(" ");
        this->ctrl->associatedWidget()->move(v[0].toInt(), v[1].toInt());
        w = v[2].toInt();
        h = v[3].toInt();
        this->ctrl->associatedWidget()->resize(w, h);
    }
}

QWidget * positionPropEditor::getEditWidget(){
    QWidget * ret = abstractPropEditor::getEditWidget();
    ret->setToolTip(tr("Set the position (x, y) and size (width, height) for the control using a 4 elemento vector and pixels as unit."));
    return ret;
}

QString positionPropEditor::generateCode()
{
    QString ret;
    ret = "'Position', [";
    ret += QString::number(this->ctrl->associatedWidget()->x()) + " ";
    ret += QString::number(this->ctrl->associatedWidget()->parentWidget()->height()  - this->ctrl->associatedWidget()->y() - this->ctrl->associatedWidget()->height()) + " ";
    ret += QString::number(this->ctrl->associatedWidget()->width()) + " ";
    ret += QString::number(this->ctrl->associatedWidget()->height()) + "]";
    return ret;
}

//////////////////////////////////////////////////////////////////////////

fontNamePropEditor::fontNamePropEditor(abstractUICCtrl *ctrl):abstractPropEditor(ctrl)
{
    this->propName = "FontName";
    this->ctrl->associatedWidget()->setFont(mainWnd::getMainWnd()->getAppConfig()->getCtrlFont());
}

QString fontNamePropEditor::getValue()
{
    return this->ctrl->associatedWidget()->font().family();
}

bool fontNamePropEditor::isValidValue(QString newVal)
{
    return fontDB.families().contains(newVal);
}

void fontNamePropEditor::setValue(QString newVal)
{
    QFont font;
    font = this->ctrl->associatedWidget()->font();
    font.setFamily(newVal);
    this->ctrl->associatedWidget()->setFont(font);
}

QString fontNamePropEditor::generateCode()
{
    return "'" + this->propName + "', '" + this->getValue() + "'";
}

QWidget * fontNamePropEditor::getEditWidget()
{
    comboUpdPropWdg * cb = new comboUpdPropWdg(this);
    cb->addItems(fontDB.families());
    //cb->setCurrentText(this->getValue());
    cb->setCurrentIndex(cb->findText(this->getValue()));
    connect(cb, SIGNAL(currentIndexChanged(QString)), this, SLOT(fontChanged(QString)));
    cb->setToolTip(tr("Sets the font to use in the control. Keep in mind that if you use an uncommon font, it may not be available on other computers or operating systems."));
    return cb;
}


void fontNamePropEditor::fontChanged(const QString & txt)
{
    comboUpdPropWdg * cb = (comboUpdPropWdg *)QObject::sender();
    if(this->isValidValue(txt))
        this->setValue(txt);
    else
    {
        disconnect(cb, SIGNAL(currentIndexChanged(QString)), this, SLOT(fontChanged(QString)));
        cb->setCurrentIndex(cb->findText(this->getValue()));
        connect(cb, SIGNAL(currentIndexChanged(QString)), this, SLOT(fontChanged(QString)));
    }
}

fontSizePropEditor::fontSizePropEditor(abstractUICCtrl *ctrl):abstractPropEditor(ctrl)
{
    this->propName = "FontSize";
}

QString fontSizePropEditor::getValue()
{
    return QString::number(this->ctrl->associatedWidget()->font().pointSize());
}

bool fontSizePropEditor::isValidValue(QString newVal)
{
    bool Ok;
    double dVal;
    dVal = newVal.toInt(&Ok);
    return (dVal > 0) && Ok;
}

void fontSizePropEditor::setValue(QString newVal)
{
    QFont font = this->ctrl->associatedWidget()->font();
    font.setPointSize(newVal.toInt());
    this->ctrl->associatedWidget()->setFont(font);
}

QString fontSizePropEditor::generateCode()
{
    QString ret;

    ret = "'" + this->propName + "', " + this->getValue() + ", ";
    ret += "'FontUnits', 'points'";
    return ret;
}

QWidget * fontSizePropEditor::getEditWidget()
{
    spinUpdPropWdg * sb = new spinUpdPropWdg(this);
    sb->setAlignment(Qt::AlignRight);
    sb->setMinimum(1);
    sb->setMaximum(5000);
    sb->setSingleStep(1);
    sb->setValue(this->getValue().toInt());
    sb->setToolTip(tr("Set font size using points as unit"));
    connect(sb, SIGNAL(valueChanged(QString)), this, SLOT(fontSizeChanged(QString)));
    return sb;
}

void fontSizePropEditor::fontSizeChanged(const QString &text)
{
    spinUpdPropWdg * sb = (spinUpdPropWdg *)QObject::sender();
    if(this->isValidValue(text))
    {
        this->setValue(text);
    }
    else
    {
        disconnect(sb, SIGNAL(valueChanged(QString)), this, SLOT(fontSizeChanged(QString)));
        sb->setValue(this->getValue().toInt());
        connect(sb, SIGNAL(valueChanged(QString)), this, SLOT(fontSizeChanged(QString)));
    }
}

fontWeightPropEditor::fontWeightPropEditor(abstractUICCtrl *ctrl):abstractPropEditor(ctrl)
{
    this->propName = "FontWeight";
}

QString fontWeightPropEditor::getValue()
{
    QString ret;
    ret = "normal";
    if(this->ctrl->associatedWidget()->font().bold())
        ret = "bold";
    return ret;
}

bool fontWeightPropEditor::isValidValue(QString newVal)
{
    return (newVal == "bold") || (newVal == "normal");
}

void fontWeightPropEditor::setValue(QString newVal)
{
    QFont font = this->ctrl->associatedWidget()->font();
    font.setBold(newVal == "bold");
    this->ctrl->associatedWidget()->setFont(font);
}

QString fontWeightPropEditor::generateCode()
{
    return "'" + this->propName + "', '" + this->getValue() + "'";
}

QWidget * fontWeightPropEditor::getEditWidget()
{
    comboUpdPropWdg * cb = new comboUpdPropWdg(this);
    cb->addItem("normal");
    cb->addItem("bold");
    //cb->setCurrentText(this->getValue());
    cb->setCurrentIndex(cb->findText(this->getValue()));
    connect(cb, SIGNAL(currentIndexChanged(QString)), this, SLOT(fontWeightChanged(QString)));
    cb->setToolTip(tr("Sets bold attribute to font used in control"));
    return cb;
}

void fontWeightPropEditor::fontWeightChanged(const QString & text)
{
    comboUpdPropWdg * cb = (comboUpdPropWdg *)QObject::sender();
    if(this->isValidValue(text))
    {
      this->setValue(text);
    }
    else
    {
        disconnect(cb, SIGNAL(currentIndexChanged(QString)), this, SLOT(fontWeightChanged(QString)));
        cb->setCurrentIndex(cb->findText(this->getValue()));
        connect(cb, SIGNAL(currentIndexChanged(QString)), this, SLOT(fontWeightChanged(QString)));
    }
}
//////////////////
fontAnglePropEditor::fontAnglePropEditor(abstractUICCtrl *ctrl):abstractPropEditor(ctrl)
{
    this->propName = "FontAngle";
}

QString fontAnglePropEditor::getValue()
{
    QString ret;
    ret = "normal";
    if(this->ctrl->associatedWidget()->font().italic())
        ret = "italic";
    return ret;
}

bool fontAnglePropEditor::isValidValue(QString newVal)
{
    return (newVal == "italic") || (newVal == "normal");
}

void fontAnglePropEditor::setValue(QString newVal)
{
    QFont font = this->ctrl->associatedWidget()->font();
    font.setItalic(newVal == "italic");
    this->ctrl->associatedWidget()->setFont(font);
}

QString fontAnglePropEditor::generateCode()
{
    return "'" + this->propName + "', '" + this->getValue() + "'";
}

QWidget * fontAnglePropEditor::getEditWidget()
{
    comboUpdPropWdg * cb = new comboUpdPropWdg(this);
    cb->addItem("normal");
    cb->addItem("italic");
    //cb->setCurrentText(this->getValue());
    cb->setCurrentIndex(cb->findText(this->getValue()));
    connect(cb, SIGNAL(currentIndexChanged(QString)), this, SLOT(fontAngleChanged(QString)));
    cb->setToolTip(tr("Sets italic attribute to font used in control"));
    return cb;
}

void fontAnglePropEditor::fontAngleChanged(const QString & text)
{
    comboUpdPropWdg * cb = (comboUpdPropWdg *)QObject::sender();
    if(this->isValidValue(text))
    {
      this->setValue(text);
    }
    else
    {
        disconnect(cb, SIGNAL(currentIndexChanged(QString)), this, SLOT(fontAngleChanged(QString)));
        cb->setCurrentIndex(cb->findText(this->getValue()));
        connect(cb, SIGNAL(currentIndexChanged(QString)), this, SLOT(fontAngleChanged(QString)));
    }
}
///////////////////
toolTipPropEditor::toolTipPropEditor(abstractUICCtrl *ctrl):abstractPropEditor(ctrl)
{
    this->propName = "TooltipString";
}

QString toolTipPropEditor::getValue()
{
    return this->toolTipString;
}

bool toolTipPropEditor::isValidValue(QString newVal)
{
    return !newVal.contains("'");
}

void toolTipPropEditor::setValue(QString newVal)
{
    this->toolTipString = newVal;
    this->ctrl->associatedWidget()->setToolTip(newVal);
}


QWidget * toolTipPropEditor::getEditWidget(){
    QWidget *ret = abstractPropEditor::getEditWidget();
    ret->setToolTip(tr("Sets the tooltip text"));
    return ret;
}


QString toolTipPropEditor::generateCode()
{
    return "'" + this->propName + "', '" + this->getValue() + "'";
}


////////////////////

visiblePropEditor::visiblePropEditor(abstractUICCtrl *ctrl):abstractPropEditor(ctrl){
    this->propName = "visible";
    this->m_visible = "on";
}

QString visiblePropEditor::getValue(){
    return this->m_visible;
}

bool visiblePropEditor::isValidValue(QString newVal){
    return (newVal.toLower() == "on") || (newVal.toLower() == "off");
}

void visiblePropEditor::setValue(QString newVal){
    if(this->isValidValue(newVal)){
        this->m_visible = newVal.toLower();
        if(this->m_visible=="on")
            this->ctrl->setHidden(false);
        else
            this->ctrl->setHidden(true);
        this->ctrl->associatedWidget()->style()->unpolish(this->ctrl->associatedWidget());
        this->ctrl->associatedWidget()->style()->polish(this->ctrl->associatedWidget());
        this->ctrl->associatedWidget()->update();
    }
}

QString visiblePropEditor::generateCode(){
    return "'" + this->propName + "', '" + this->getValue() + "'";
}

QWidget * visiblePropEditor::getEditWidget(){
    comboUpdPropWdg * cb = new comboUpdPropWdg(this);
    cb->addItem("on");
    cb->addItem("off");
    cb->setCurrentIndex(cb->findText(this->getValue()));
    connect(cb, SIGNAL(currentIndexChanged(QString)), this, SLOT(visibleChanged(QString)));
    cb->setToolTip(tr("Make visible or not a control at runtime. In the editor, the control will be ever visible, but will be marked with e red dashed box."));
    return cb;
}

void visiblePropEditor::visibleChanged(const QString & txt){
    comboUpdPropWdg * cb = (comboUpdPropWdg *)QObject::sender();
    if(this->isValidValue(txt)){
        this->setValue(txt);
    }
    else
    {
        disconnect(cb, SIGNAL(currentIndexChanged(QString)), this, SLOT(visibleChanged(QString)));
        cb->setCurrentIndex(cb->findText(this->getValue()));
        connect(cb, SIGNAL(currentIndexChanged(QString)), this, SLOT(visibleChanged(QString)));
    }
}

defValuePropEditor::defValuePropEditor(abstractUICCtrl *ctrl):abstractPropEditor(ctrl)
{
    this->propName = "Value";
    this->defValue = 1;
}

QString defValuePropEditor::getValue()
{
    return QString::number(this->defValue);
}

bool defValuePropEditor::isValidValue(QString newVal)
{
    bool Ok;
    double dVal;
    dVal = newVal.toInt(&Ok);
    return (dVal >= 0) && Ok ;
}

void defValuePropEditor::setValue(QString newVal)
{
    if(this->isValidValue(newVal))
        this->defValue = newVal.toInt();
}

QString defValuePropEditor::generateCode()
{
    QString ret;
    ret = "'" + this->propName + "', " + this->getValue();
    return ret;
}

QWidget * defValuePropEditor::getEditWidget()
{
    spinUpdPropWdg * sb = new spinUpdPropWdg(this);
    sb->setAlignment(Qt::AlignRight);
    sb->setMinimum(0);
    sb->setMaximum(1000);
    sb->setSingleStep(1);
    sb->setValue(this->getValue().toInt());
    sb->setToolTip(tr("Sets the preselected item. Must be a value between 1 and the number of options available."));
    connect(sb, SIGNAL(valueChanged(QString)), this, SLOT(defValueChanged(QString)));
    return sb;
}

void defValuePropEditor::defValueChanged(const QString &text)
{
    spinUpdPropWdg * sb = (spinUpdPropWdg *)QObject::sender();
    if(this->isValidValue(text))
    {
        this->setValue(text);
    }
    else
    {
        disconnect(sb, SIGNAL(valueChanged(QString)), this, SLOT(defValueChanged(QString)));
        sb->setValue(this->getValue().toInt());
        connect(sb, SIGNAL(valueChanged(QString)), this, SLOT(defValueChanged(QString)));
    }
}


tagPropEditor::tagPropEditor(abstractUICCtrl *ctrl):abstractPropEditor(ctrl)
{
    this->propName = "tag";
    this->value = "";
}

QString tagPropEditor::getValue(){
    return this->value;
}

bool tagPropEditor::isValidValue(QString newVal){
    return !newVal.contains("'");
}

void tagPropEditor::setValue(QString newVal){
    this->value = newVal;
}

QWidget * tagPropEditor::getEditWidget(){
    QWidget * ret = abstractPropEditor::getEditWidget();
    ret->setToolTip(tr("Object identificator for used with de function findobj"));
    return ret;
}

userDataPropEditor::userDataPropEditor(abstractUICCtrl *ctrl):abstractPropEditor(ctrl)
{
    this->propName = "userData";
    this->value = "";
}

QString userDataPropEditor::getValue(){
    return this->value;
}

bool userDataPropEditor::isValidValue(QString newVal){
    return !newVal.contains("'");
}

void userDataPropEditor::setValue(QString newVal){
    this->value = newVal;
}

QWidget * userDataPropEditor::getEditWidget(){
    QWidget * ret = abstractPropEditor::getEditWidget();
    ret->setToolTip(tr("User defined data to associate with the graphics object. Can be used to share data between controls"));
    return ret;
}

vertAlignPropEditor::vertAlignPropEditor(abstractUICCtrl *ctrl):abstractPropEditor(ctrl)
{
    this->propName = "verticalAlignment";
    this->vAlign = "middle";
}

QString vertAlignPropEditor::getValue()
{
    return this->vAlign;
}

bool vertAlignPropEditor::isValidValue(QString newVal)
{
    return (newVal == "bottom") || (newVal == "middle") || (newVal == "top");
}

void vertAlignPropEditor::setValue(QString newVal)
{
    if(this->isValidValue(newVal))
        this->vAlign = newVal;
}

QWidget * vertAlignPropEditor::getEditWidget()
{
    comboUpdPropWdg * cb = new comboUpdPropWdg(this);
    cb->addItem("top");
    cb->addItem("middle");
    cb->addItem("bottom");
    cb->setCurrentIndex(cb->findText(this->getValue()));
    connect(cb, SIGNAL(currentIndexChanged(QString)), this, SLOT(vAlignValueChanged(QString)));
    cb->setToolTip(tr("Specifies the vertical position of the text in the uicontrol object"));
    return cb;
}

void vertAlignPropEditor::vAlignValueChanged(const QString &text)
{
    comboUpdPropWdg * cb = (comboUpdPropWdg *)QObject::sender();
    if(this->isValidValue(text)){
        this->setValue(text);
    }
    else
    {
        disconnect(cb, SIGNAL(currentIndexChanged(QString)), this, SLOT(vAlignValueChanged(QString)));
        cb->setCurrentIndex(cb->findText(this->getValue()));
        connect(cb, SIGNAL(currentIndexChanged(QString)), this, SLOT(vAlignValueChanged(QString)));
    }
}
