/****************************************************************************
Copyright 2015-2020, Enrique Sergio Burgos

This file is part of GUI Editor for Octave.

GUI Editor for Octave is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

GUI Editor is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GUI Editor.  If not, see <http://www.gnu.org/licenses/>.
****************************************************************************/
#include "imgctrl.h"
#include "editableimg.h"
#include "storagemanager.h"
#include <QFile>
#include <QTextStream>
#include <QDir>
#include "wdgclrchooser.h"

imgPosPropEditor::imgPosPropEditor(abstractUICCtrl *ctrl):abstractPropEditor(ctrl)
{
    this->propName = "Position";
}

QString imgPosPropEditor::getValue()
{
    QString ret;
    ret = "[";
    ret += QString::number(this->ctrl->associatedWidget()->x()) + " ";
    ret += QString::number(this->ctrl->associatedWidget()->y()) + " ";
    ret += QString::number(this->ctrl->associatedWidget()->width()) + " ";
    ret += QString::number(this->ctrl->associatedWidget()->height()) + "]";
    return ret;
}

bool imgPosPropEditor::isValidValue(QString newVal)
{
    QRegExp r("^\\[[-+]?\\d{1,10}\\s[-+]?\\d{1,10}\\s\\d{1,10}\\s\\d{1,10}\\]$");
    return newVal.contains(r) == 1;
}

void imgPosPropEditor::setValue(QString newVal)
{
    QStringList v;
    int w, h;
    if(this->isValidValue(newVal))
    {
        newVal.remove("[");
        newVal.remove("]");
        v = newVal.split(" ");
        this->ctrl->associatedWidget()->move(v[0].toInt(), v[1].toInt());
        w = v[2].toInt();
        h = v[3].toInt();
        this->ctrl->associatedWidget()->resize(w, h);
    }
}

QWidget * imgPosPropEditor::getEditWidget(){
    QWidget * ret = abstractPropEditor::getEditWidget();
    ret->setToolTip(tr("Set the position (x, y) and size (width, height) for the control using a 4 elemento vector and pixels as unit."));
    return ret;
}

QString imgPosPropEditor::generateCode()
{
    QString ret;
    QString posCmd;
    posCmd = this->ctrl->properties.getValueByName("positionDef");
    ret = "'"+ posCmd +"', [";
    ret += QString::number(this->ctrl->associatedWidget()->x()) + " ";
    ret += QString::number(this->ctrl->associatedWidget()->parentWidget()->height()  - this->ctrl->associatedWidget()->y() - this->ctrl->associatedWidget()->height()) + " ";
    ret += QString::number(this->ctrl->associatedWidget()->width()) + " ";
    ret += QString::number(this->ctrl->associatedWidget()->height()) + "]";
    return ret;
}


postionDefPropEditor::postionDefPropEditor(abstractUICCtrl *ctrl):abstractPropEditor(ctrl)
{
    this->positionType = "position";
    this->propName = "positionDef";
}
QString postionDefPropEditor::getValue(){
    return this->positionType;
}
bool postionDefPropEditor::isValidValue(QString newVal){
    bool ret = false;
    if((newVal == "position") || (newVal == "outerposition")){
        ret = true;
    }
    return ret;
}

void postionDefPropEditor::setValue(QString newVal){
    if((newVal == "position") || (newVal == "outerposition")){
        this->positionType = newVal;
    }
}

QWidget * postionDefPropEditor::getEditWidget(){
    comboUpdPropWdg * cb = new comboUpdPropWdg(this);
    cb->addItem("position");
    cb->addItem("outerposition");
    cb->setCurrentIndex(cb->findText(this->getValue()));
    cb->setToolTip(tr("Use 'position' or 'outerposition' property to set size and position."));
    connect(cb, SIGNAL(currentIndexChanged(QString)), this, SLOT(valueChooserChanged(QString)));
    return cb;
}

void postionDefPropEditor::valueChooserChanged(QString text){
    comboUpdPropWdg * cb = (comboUpdPropWdg *)QObject::sender();
    if(this->isValidValue(text))
        this->setValue(text);
    else
    {
        disconnect(cb, SIGNAL(currentIndexChanged(QString)), this, SLOT(valueChooserChanged(QString)));
        cb->setCurrentIndex(cb->findText(this->getValue()));
        connect(cb, SIGNAL(currentIndexChanged(QString)), this, SLOT(valueChooserChanged(QString)));
    }
}
unsigned int imgCtrl::uicNameCounter = 0;

unsigned int imgCtrl::getNameCounter()
{
    return uicNameCounter;
}

imgCtrl::imgCtrl(QWidget *parent, abstractUICCtrl *octaveParent, childWndDlg * parentWnd)
    :abstractUICCtrl(parent, octaveParent, parentWnd)
{
    editableImage * img = new editableImage(parent, this);
    uicNameCounter++;
    this->setCtrlName("Image_"+QString::number(uicNameCounter));
    img->resize(80, 80);

    this->setAssociatedWidget(img);
    this->adjTL->asociateCtrl(this->getAssociatedWidget());
    this->adjML->asociateCtrl(this->getAssociatedWidget());
    this->adjBL->asociateCtrl(this->getAssociatedWidget());
    this->adjTR->asociateCtrl(this->getAssociatedWidget());
    this->adjMR->asociateCtrl(this->getAssociatedWidget());
    this->adjBR->asociateCtrl(this->getAssociatedWidget());
    this->adjTM->asociateCtrl(this->getAssociatedWidget());
    this->adjBM->asociateCtrl(this->getAssociatedWidget());
    this->properties.registerProp(new imgPosPropEditor(this));
    this->properties.registerProp(new fontNamePropEditor(this));
    this->properties.registerProp(new fontSizePropEditor(this));
    this->properties.registerProp(new fontWeightPropEditor(this));
    this->properties.registerProp(new fontAnglePropEditor(this));
    this->properties.registerProp(new visiblePropEditor(this));
    this->properties.registerProp(new postionDefPropEditor(this));

    QStringList defSrc;
    callBackEditor * cb;

    defSrc.clear();
    cb = new callBackEditor(this, "buttondownfcn");
    defSrc.append("% ");
    defSrc.append(QObject::tr("% This code will be executed when user make  a click over the control"));
    defSrc.append("% ");
    defSrc.append("");
    cb->setDefSrc(defSrc);
    this->callbacks.registerCB(cb);

    defSrc.clear();
    cb = new callBackEditor(this, "deletefcn");
    defSrc.append("% ");
    defSrc.append(QObject::tr("% Callback function executed immediately before uicontrol is deleted. "));
    defSrc.append("% ");
    defSrc.append("");
    cb->setDefSrc(defSrc);
    this->callbacks.registerCB(cb);
}

void imgCtrl::setParents(QWidget * parent, abstractUICCtrl * octaveParent, childWndDlg * parentWnd)
{
    editableImage * img = dynamic_cast<editableImage *>(this->associatedWidget());


    this->adjTL->setParent(parent);
    this->adjML->setParent(parent);
    this->adjBL->setParent(parent);
    this->adjTR->setParent(parent);
    this->adjMR->setParent(parent);
    this->adjBR->setParent(parent);
    this->adjTM->setParent(parent);
    this->adjBM->setParent(parent);

    img->setParents(parent, this);

    this->setOctaveParent(octaveParent);
    this->setParentWnd(parentWnd);
}

QStringList imgCtrl::generateMFile(QString path)
{
    QStringList ret;
    QString lineMCode;
    abstractPropEditor * prop;
    int i;
    ret.append("  " + this->properties.getValueByName("Name") + " = axes( ...") ;
    ret.append("\t'Units', 'pixels'");
    if(getOctaveParent())
      ret.append("\t'parent'," + getOctaveParent()->getCtrlName());
    prop = this->properties.getFirst();
    while(prop)
    {
        if(prop->canGenerateCode())
        {
          lineMCode = "\t" + prop->generateCode();
          ret.append(lineMCode);
        }
        prop = this->properties.getNext();
    }
    for(i = 1; i < ret.count() - 1; i++)
        ret[i] = ret[i] + ", ... ";
    ret[i] = ret[i] + ");";
    this->createCallBack(path);
    return ret;
}

abstractUICCtrl * imgCtrl::clone(QWidget *parent,
                                 abstractUICCtrl *octaveParent,
                                 childWndDlg * parentWnd){
    callBackEditor * cb;
    callBackEditor *rCb;
    imgCtrl * ret = new imgCtrl(parent,
                                octaveParent,
                                parentWnd);
    abstractPropEditor *prop =  this->properties.getFirst();
    while(prop){
        ret->properties.getPropByName(prop->getPropName())->setValue(prop->getValue());
        prop = this->properties.getNext();
    }
    // La siguiente línea es reemplazada por el fragmento que sigue.
    //ret->srcCallBack = this->srcCallBack;
    cb = this->callbacks.getFirst();
    while(cb){
        rCb = ret->callbacks.getCbByName(cb->getCallBackName());
        /* No puede suceder que no exista un callback con el nombre dado
           porque se trata de un objeto del mismo tipo, pero por seguridad
           se hace la comprobación.
        */
        if(rCb){
          rCb->setActive(cb->isActive());
          rCb->setSrc(cb->getSrcCode());
        }
        cb = this->callbacks.getNext();
    }
    // Fin copia de los callbacks
    return ret;
}

void imgCtrl::setHidden(bool isHidden){
    ((editableImage *)this->associatedWidget())->setHidden(isHidden);
}

imgCtrl::~imgCtrl()
{
  delete this->getAssociatedWidget();
}

imgCtrlGen::imgCtrlGen():controlGenerator("imgCtrl")
{
}

abstractUICCtrl * imgCtrlGen::getControl(QXmlStreamReader &xml, abstractUICCtrl *parent, childWndDlg * dlg)
{
    QString src;
    imgCtrl * img = new imgCtrl(parent->getAssociatedWidget(), parent, dlg);
    abstractPropEditor * prop;
    prop = img->properties.getFirst();
    while(prop)
    {
      prop->setValue(xml.attributes().value(prop->getPropName()).toString());
      prop = img->properties.getNext();
    }
    img->deselect();
    return img;

}

imgCtrlGenV1::imgCtrlGenV1():controlGenerator("imgCtrl")
{
}

abstractUICCtrl * imgCtrlGenV1::getControl(QXmlStreamReader &xml, abstractUICCtrl *parent, childWndDlg * dlg)
{
    QString src;
    imgCtrl * img = new imgCtrl(parent->getAssociatedWidget(), parent, dlg);
    abstractPropEditor * prop;
    prop = img->properties.getFirst();
    while(prop)
    {
      prop->setValue(xml.attributes().value(prop->getPropName()).toString());
      prop = img->properties.getNext();
    }
    bool cbActive = xml.attributes().value("GenerateCallBack").toString() == "true";
    src = xml.readElementText();
    if(src.length() > 0){
        callBackEditor *cb = img->callbacks.getCbByName("callback");
        if(cb){
            cb->setSrc(src.split('\n'));
            cb->setActive(cbActive);
        }
    }
    img->deselect();
    return img;
}

