/****************************************************************************
Copyright 2015-2020, Enrique Sergio Burgos

This file is part of GUI Editor for Octave.

GUI Editor for Octave is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

GUI Editor is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GUI Editor.  If not, see <http://www.gnu.org/licenses/>.
****************************************************************************/
#include "storagemanager.h"
#include <QFile>
#include <QXmlStreamReader>
#include <QXmlStreamWriter>
#include "buttonctrl.h"
#include "editctrl.h"
#include "textctrl.h"
#include "cmbctrl.h"
#include "togglectrl.h"
#include "checkboxctrl.h"
#include "radiobuttonctrl.h"
#include "framectrl.h"
#include "listctrl.h"
#include "sliderctrl.h"
#include "imgctrl.h"
#include "ananoctrl.h"
#include "nodeMCUctrl.h"
#include "eduCIAACtrl.h"
#include "grouppanelctrl.h"
#include "callbackctrl.h"
#include "buttongroupctrl.h"
#include "tablectrl.h"
#include <QFileInfo>
#include <QDebug>
#include "mainwnd.h"
#include "ui_mainwnd.h"

QList<controlGenerator *>& storageManager::getGeneratorList()
{
    static QList<controlGenerator *>  generatorList;
    return generatorList;
}

QList<controlGenerator *>& storageManager::getGeneratorListV1()
{
    static QList<controlGenerator *>  generatorListV1;
    return generatorListV1;
}

void storageManager::loadCallBacks(abstractUICCtrl * ctrl, QXmlStreamReader &xml){
    QString src;
    callBackEditor * cb;
    while(xml.readNextStartElement())
    {
        cb = ctrl->callbacks.getCbByName(xml.attributes().value("name").toString());
        if(cb){
            cb->setActive(xml.attributes().value("active").toString() == "true");
            src = xml.readElementText();
            if(src.length() > 0)
                cb->setSrc(src.split('\n'));
        }
    }
}

int storageManager::registeredGeneratosCount()
{
    return getGeneratorList().count();
}

int storageManager::registeredGeneratosCountV1()
{
    return getGeneratorListV1().count();
}

void storageManager::registerGenerators()
{
    getGeneratorList().push_back(new buttonCtrlGen());
    getGeneratorList().push_back(new editCtrlGen());
    getGeneratorList().push_back(new textCtrlGen());
    getGeneratorList().push_back(new rbCtrlGen());
    getGeneratorList().push_back(new cbCtrlGen());
    getGeneratorList().push_back(new cmbCtrlGen());
    getGeneratorList().push_back(new listCtrlGen());
    getGeneratorList().push_back(new frameCtrlGen());
    getGeneratorList().push_back(new toggleCtrlGen());
    getGeneratorList().push_back(new sliderCtrlGen());
    getGeneratorList().push_back(new imgCtrlGen());
    getGeneratorList().push_back(new aNanoCtrlGen());
    getGeneratorList().push_back(new nodeMCUCtrlGen());
    getGeneratorList().push_back(new eduCIAACtrlGen());
    getGeneratorList().push_back(new groupPanelCtrlGen());
    getGeneratorList().push_back(new callBackCtrlCtrlGen());
    getGeneratorList().push_back(new buttonGroupCtrlGen());
    getGeneratorList().push_back(new tableCtrlGen());
}

void storageManager::registerGeneratorsV1()
{
    getGeneratorListV1().push_back(new buttonCtrlGenV1());
    getGeneratorListV1().push_back(new editCtrlGenV1());
    getGeneratorListV1().push_back(new textCtrlGenV1());
    getGeneratorListV1().push_back(new rbCtrlGenV1());
    getGeneratorListV1().push_back(new cbCtrlGenV1());
    getGeneratorListV1().push_back(new cmbCtrlGenV1());
    getGeneratorListV1().push_back(new listCtrlGenV1());
    getGeneratorListV1().push_back(new frameCtrlGenV1());
    getGeneratorListV1().push_back(new toggleCtrlGenV1());
    getGeneratorListV1().push_back(new sliderCtrlGenV1());
    getGeneratorListV1().push_back(new imgCtrlGenV1());
    getGeneratorListV1().push_back(new aNanoCtrlGenV1());
    getGeneratorListV1().push_back(new nodeMCUCtrlGenV1());
    getGeneratorListV1().push_back(new eduCIAACtrlGenV1());
    getGeneratorListV1().push_back(new groupPanelCtrlGenV1());
    getGeneratorListV1().push_back(new callBackCtrlCtrlGenV1());
    getGeneratorListV1().push_back(new buttonGroupCtrlGenV1());
    getGeneratorListV1().push_back(new tableCtrlGenV1());
}

void storageManager::unregisterGenerators()
{
    QList<controlGenerator *>::iterator it;
    for(it = getGeneratorList().begin(); it != getGeneratorList().end(); it++)
        delete *it;
    getGeneratorList().clear();
}

void storageManager::unregisterGeneratorsV1()
{
    QList<controlGenerator *>::iterator it;
    for(it = getGeneratorListV1().begin(); it != getGeneratorListV1().end(); it++)
        delete *it;
    getGeneratorListV1().clear();
}

void storageManager::loadDlgFromFileRecursive(childWndDlg * dlg,
                                              abstractUICCtrl * octaveParent,
                                              QXmlStreamReader &xml)
{
    QList<controlGenerator *>::iterator it;
    abstractUICCtrl * ctrl;
    QPalette palette;
    QString src;
    while(xml.readNextStartElement())
    {
        if(xml.name() == "uiControl")
        {
            it = getGeneratorList().begin();
            ctrl = NULL;
            while((it != getGeneratorList().end()) && (!ctrl))
            {
                if(xml.attributes().value("class").toString() == (*it)->getClassName())
                {
                    ctrl = (*it)->getControl(xml, octaveParent, dlg);
                    ctrl->setOctaveParent(octaveParent);
                    octaveParent->addWidget(ctrl);
                    if(xml.readNextStartElement())
                        if(xml.name().toString() == "callbacks")
                            storageManager::loadCallBacks(ctrl, xml);
                    if(ctrl->isContainer()){
                        loadDlgFromFileRecursive(dlg, ctrl, xml);
                    }
                    else
                        xml.readNextStartElement();
                }
                it ++;
            }
        }
    }
}

void storageManager::loadDlgFromFileRecursiveV1(childWndDlg * dlg,
                                              abstractUICCtrl * octaveParent,
                                              QXmlStreamReader &xml)
{
    QList<controlGenerator *>::iterator it;
    abstractUICCtrl * ctrl;
    QXmlStreamReader::TokenType readedToken;
    readedToken = xml.readNext();
    int x = 0;
    while(!xml.atEnd())
    {
        if(readedToken == QXmlStreamReader::StartElement)
        {
            if(xml.name() == "uiControl")
            {
                x++;
                it = getGeneratorListV1().begin();
                ctrl = NULL;
                while((it != getGeneratorListV1().end()) && (!ctrl))
                {
                    if(xml.attributes().value("class").toString() == (*it)->getClassName())
                    {
                        ctrl = (*it)->getControl(xml, octaveParent, dlg);
                        ctrl->setOctaveParent(octaveParent);
                        octaveParent->addWidget(ctrl);
                        if(ctrl->isContainer())
                        {
                            loadDlgFromFileRecursiveV1(dlg, ctrl, xml);
                        }
                    }
                    it ++;
                }
            }
            readedToken = xml.tokenType();
        }
        else if(readedToken == QXmlStreamReader::EndElement)
        {
            if (x == 0)
                return;
            x--;
            readedToken = xml.readNext();

        }
        else
          readedToken = xml.readNext();
    }
}

void storageManager::loadDlgFromFile(childWndDlg * dlg, QString name)
{
    callBackEditor * cb;
    QList<controlGenerator *>::iterator it;
    QFile f(name);
    QXmlStreamReader xml;
    abstractPropEditor * prop;
    QString src;
    abstractUICCtrl *ctrl;
    QPalette palette;
    frameDlg * frm = dlg->getFrame();
    f.open(QIODevice::ReadOnly);
    xml.setDevice(&f);
    if(storageManager::registeredGeneratosCount() == 0)
        storageManager::registerGenerators();

    if(xml.readNextStartElement()){
        if(xml.name().toString() == "dialog"){
            prop = frm->properties.getFirst();
            while(prop){
                prop->setValue(xml.attributes().value(prop->getPropName()).toString());
                prop = frm->properties.getNext();
            }
            if(xml.readNextStartElement()){
                /* Si es una versión anterior, el siguiente elemento puede ser GlobalSrc o un uiControl*/
                /* en la nueva versión puede ser callbacks y la lista de entradas de callbacks */
                if(xml.name().toString()=="GlobalSrc"){
                    src = xml.readElementText();
                    if(src.length() > 0){
                        cb = frm->callbacks.getCbByName("onload");
                        if(cb){
                            cb->setSrc(src.split('\n'));
                        }
                    }
                }
                else if(xml.name().toString() == "callbacks"){
                    storageManager::loadCallBacks(frm, xml);
                }
                while(xml.readNextStartElement()){
                    if(xml.name().toString() == "uiControl")
                    {
                        it = getGeneratorList().begin();
                        ctrl = NULL;
                        while((it != getGeneratorList().end()) && (!ctrl))
                        {
                            if(xml.attributes().value("class").toString() == (*it)->getClassName())
                            {
                                ctrl = (*it)->getControl(xml, frm, dlg);
                                frm->addWidget(ctrl);
                                palette = ctrl->associatedWidget()->palette();
                                if(xml.readNextStartElement())
                                    if(xml.name().toString() == "callbacks")
                                        storageManager::loadCallBacks(ctrl, xml);
                                if(ctrl->isContainer()){
                                    loadDlgFromFileRecursive(dlg, ctrl, xml);
                                }
                                else
                                    xml.readNextStartElement();
                            }
                            it ++;
                        }
                    }
                }
            }
        }
    }

    storageManager::unregisterGenerators();
    f.close();
    dlg->documentHaveName = true;
    dlg->documentName = name;
    QFileInfo fi(name);
    dlg->setWindowTitle(fi.fileName());
    QString size = dlg->getFrame()->properties.getPropByName("Position")->getValue();
    size.remove("[");
    size.remove("]");
    QStringList sizeParts = size.split(" ");
    if(sizeParts.count() == 4){
        dlg->move(0, 0);
        dlg->resize(sizeParts[2].toInt(), sizeParts[3].toInt());
        dlg->updateGeometry();
        dlg->show();
        dlg->hide();
    }
}

void storageManager::loadDlgFromFileV1(childWndDlg * dlg, QString name)
{
    callBackEditor *cb;
    QList<controlGenerator *>::iterator it;
    QFile f(name);
    QXmlStreamReader xml;
    abstractPropEditor * prop;
    QString globalSrc;
    abstractUICCtrl *ctrl;
    QPalette palette;
    frameDlg * frm = dlg->getFrame();
    QXmlStreamReader::TokenType readedToken;
    f.open(QIODevice::ReadOnly);
    xml.setDevice(&f);
    if(storageManager::registeredGeneratosCountV1() == 0)
        storageManager::registerGeneratorsV1();

    readedToken = xml.readNext();
    while(!xml.atEnd())
    {
        if(readedToken == QXmlStreamReader::StartElement)
        {
            if(xml.name() == "dialog")
            {
                prop = frm->properties.getFirst();
                while(prop)
                {
                    prop->setValue(xml.attributes().value(prop->getPropName()).toString());
                    prop = frm->properties.getNext();
                }
            }
            else if(xml.name() == "GlobalSrc")
            {
                globalSrc = xml.readElementText();
                if(globalSrc.length() > 0){
                  cb = frm->callbacks.getCbByName("onload");
                  if(cb){
                      cb->setSrc(globalSrc.split('\n'));
                      cb->setActive(true);
                  }
                }

            }
            else if(xml.name() == "uiControl")
            {
                it = getGeneratorListV1().begin();
                ctrl = NULL;
                while((it != getGeneratorListV1().end()) && (!ctrl))
                {
                    if(xml.attributes().value("class").toString() == (*it)->getClassName())
                    {
                        ctrl = (*it)->getControl(xml, frm, dlg);

                        frm->addWidget(ctrl);
                        palette = ctrl->associatedWidget()->palette();

                        if(ctrl->isContainer())
                            loadDlgFromFileRecursiveV1(dlg, ctrl, xml);
                    }
                    it ++;
                }
            }
        }
        readedToken = xml.readNext();
    }
    storageManager::unregisterGeneratorsV1();
    f.close();
    dlg->documentHaveName = true;
    dlg->documentName = name;
    QFileInfo fi(name);
    dlg->setWindowTitle(fi.fileName());
    QString size = dlg->getFrame()->properties.getPropByName("Position")->getValue();
    size.remove("[");
    size.remove("]");
    QStringList sizeParts = size.split(" ");
    if(sizeParts.count() == 4){
        dlg->move(0, 0);
        dlg->resize(sizeParts[2].toInt(), sizeParts[3].toInt());
        dlg->updateGeometry();
        dlg->show();
        dlg->hide();
    }
}

void storageManager::saveDlgToFileRecursive(abstractUICCtrl *ctrl, QXmlStreamWriter &xml)
{
    callBackEditor * cb;
    QString srcLine;
    abstractUICCtrl *ctrToSave;
    abstractPropEditor * prop;
    ctrToSave = ctrl->getFirstCtrl();
    while(ctrToSave)
    {
        xml.writeStartElement("uiControl");
        xml.writeAttribute("class", ctrToSave->className());
        prop = ctrToSave->properties.getFirst();
        while(prop)
        {
            xml.writeAttribute(prop->getPropName(), prop->getValue());
            prop = ctrToSave->properties.getNext();
        }
        xml.writeStartElement("callbacks");
        cb = ctrToSave->callbacks.getFirst();
        while(cb){
            if(cb->mustSaveSrc()){
                xml.writeStartElement("callback");
                xml.writeAttribute("name", cb->getCallBackName());
                if(cb->isActive())
                    xml.writeAttribute("active", "true");
                else
                    xml.writeAttribute("active", "false");
                for(int i=0; i < cb->getSrcCode().count(); i++)
                {
                    srcLine = cb->getSrcCode()[i];
                    if(srcLine.contains('\r'))
                        srcLine.remove('\r');
                    if(srcLine.contains('\n'))
                        srcLine.remove('\n');
                    if(i < cb->getSrcCode().count() - 1)
                        srcLine = srcLine + "\n";
                    xml.writeCharacters(srcLine);
                }
                xml.writeEndElement();
            }
            cb = ctrToSave->callbacks.getNext();
        }
        xml.writeEndElement();
        if(ctrToSave->isContainer())
            saveDlgToFileRecursive(ctrToSave, xml);

        xml.writeEndElement();

        ctrToSave = ctrl->getNextCtrl();
    }
}

void storageManager::saveDlgToFile(childWndDlg *dlg, QString fileName)
{
    int i;
    callBackEditor * cb;
    QString srcLine;
    QXmlStreamWriter xml;
    abstractUICCtrl *ctrl;
    abstractPropEditor * prop;
    if(dlg)
    {
        frameDlg * frm = dlg->getFrame();
        QFile f(fileName);
        f.open(QIODevice::WriteOnly);
        xml.setDevice(&f);
        xml.setAutoFormatting(true);

        xml.writeStartDocument();

        xml.writeStartElement("dialog");
        prop = frm->properties.getFirst();
        while(prop)
        {
            xml.writeAttribute(prop->getPropName(), prop->getValue());
            prop = frm->properties.getNext();
        }

        cb = frm->callbacks.getFirst();
        xml.writeStartElement("callbacks");
        while(cb){
            if(cb->mustSaveSrc())
            {
                xml.writeStartElement("callback");
                xml.writeAttribute("name", cb->getCallBackName());
                if(cb->isActive())
                    xml.writeAttribute("active", "true");
                else
                    xml.writeAttribute("active", "false");
                for(i=0; i < cb->getSrcCode().count(); i++)
                {
                    srcLine = cb->getSrcCode()[i];
                    if(srcLine.contains('\r'))
                        srcLine.remove('\r');
                    if(srcLine.contains('\n'))
                        srcLine.remove('\n');
                    if(i < cb->getSrcCode().count() - 1)
                        srcLine = srcLine + "\n";
                    xml.writeCharacters(srcLine);
                }
                xml.writeEndElement();
            }
            cb = frm->callbacks.getNext();
        }
        xml.writeEndElement();


        ctrl = frm->getFirstCtrl();
        while(ctrl)
        {
            if(ctrl != frm)
            {
                xml.writeStartElement("uiControl");
                xml.writeAttribute("class", ctrl->className());
                prop = ctrl->properties.getFirst();
                while(prop)
                {
                    xml.writeAttribute(prop->getPropName(), prop->getValue());
                    prop = ctrl->properties.getNext();
                }

                cb = ctrl->callbacks.getFirst();
                xml.writeStartElement("callbacks");
                while(cb){
                    if(cb->mustSaveSrc()){
                        xml.writeStartElement("callback");
                        xml.writeAttribute("name", cb->getCallBackName());
                        xml.writeAttribute("class", cb->getCallBackName());
                        if(cb->isActive())
                            xml.writeAttribute("active", "true");
                        else
                            xml.writeAttribute("active", "false");
                        for(i=0; i < cb->getSrcCode().count(); i++)
                        {
                            srcLine = cb->getSrcCode()[i];
                            srcLine.remove('\n');
                            srcLine.remove('\r');
                            if(i < cb->getSrcCode().count() - 1)
                                srcLine = srcLine + "\n";
                            xml.writeCharacters(srcLine);
                        }
                        xml.writeEndElement();
                    }
                    cb = ctrl->callbacks.getNext();
                }
                xml.writeEndElement();
                if(ctrl->isContainer())
                    saveDlgToFileRecursive(ctrl, xml);
                xml.writeEndElement();
            }
            ctrl = frm->getNextCtrl();
        }
        xml.writeEndElement();
        xml.writeEndDocument();

        f.close();
    }
}

void storageManager::saveProject(guiProject * prj)
{
    QString fileName = prj->fullPath() + ".prj";
    mainWnd::getAppConfig()->registerRecentPrj(fileName);
    mainWnd::getMainWnd()->updateRecentPrj();
    QFile f(fileName);
    int vA, vB, vC;
    QXmlStreamWriter xml;
    int i;
    f.open(QIODevice::WriteOnly | QIODevice::Text);
    xml.setDevice(&f);
    xml.setAutoFormatting(true);
    xml.writeStartDocument();
    prj->version(vA, vB, vC);
    xml.writeStartElement("project");
    xml.writeAttribute("version", QString::number(vA));
    xml.writeAttribute("revision", QString::number(vB));
    xml.writeAttribute("subrevision", QString::number(vC));
    /*
     * Por defecto, todos los proyectos se guardan con la última versión
     * que actualmente es la versión 2.
    */
    prj->setFileVersion(2);
    xml.writeAttribute("fileVersion", QString::number(prj->fileVersion()));

    xml.writeAttribute("runArguments",  prj->runArguments());
    xml.writeAttribute("defineBasePath", prj->defBasePath()?"true":"false");
    xml.writeAttribute("defineImagePath", prj->defImgPath()?"true":"false");

    if(prj->getMainDlg())
        xml.writeAttribute("startDlg", prj->getMainDlg()->getName());

    for(i = 0; i < prj->searchPath().count(); i++)
        xml.writeAttribute("searchPath_" + QString::number(i), prj->searchPath()[i]);



    xml.writeStartElement("warning");
    xml.writeAttribute("disableAll", prj->disableAllWarnings ? "true" : "false");

    for(int k = 0; k < prj->disabledWarningMessages.count(); k++){
        xml.writeStartElement("disabledWarnings");
        xml.writeAttribute("message", prj->disabledWarningMessages[k]);
        xml.writeEndElement();
    }
    xml.writeEndElement();
    xml.writeEndElement();
    xml.writeEndDocument();
    f.close();
}

guiProject * storageManager::loadProject(QString fileName)
{
    QFileInfo projectInfo(fileName);
    int vA, vB, vC;
    int i;
    QString pathToAdd;
    QString mainDlgName = "mainDlg";
    QStringList searchPath;
    QString prjPath  = projectInfo.absolutePath();
    QString prjFileName = projectInfo.fileName().replace(".prj", "");
    guiProject * ret = new guiProject(prjPath, prjFileName, mainWnd::getMainWnd()->ui->mdiArea);

    QFile f(fileName);
    QXmlStreamReader xml;
    f.open(QIODevice::ReadOnly);
    xml.setDevice(&f);
    QXmlStreamReader::TokenType readedToken;

    readedToken = xml.readNext();
    while(!xml.atEnd())
    {
        if(readedToken == QXmlStreamReader::StartElement)
        {
            if(xml.name() == "project")
            {
                vA = xml.attributes().value("version").toInt();
                vB = xml.attributes().value("revision").toInt();
                vC = xml.attributes().value("subrevision").toInt();
                if(xml.attributes().hasAttribute("fileVersion"))
                    ret->setFileVersion(xml.attributes().value("fileVersion").toInt());
                else
                    ret->setFileVersion(1);
                ret->setVersion(vA, vB, vC);
                ret->setRunArguments(xml.attributes().value("runArguments").toString());
                i = 0;
                while(xml.attributes().hasAttribute("searchPath_" + QString::number(i)))
                {
                    pathToAdd = xml.attributes().value("searchPath_" + QString::number(i)).toString();
                    searchPath.push_back(pathToAdd);
                    i++;
                }
                ret->setSearchPath(searchPath);
                if(xml.attributes().hasAttribute("startDlg"))
                    mainDlgName = xml.attributes().value("startDlg").toString();

                if(xml.attributes().hasAttribute("defineBasePath"))
                    ret->setDefBasePath((xml.attributes().value("defineBasePath") == "true"));

                if(xml.attributes().hasAttribute("defineImagePath"))
                    ret->setDefBasePath((xml.attributes().value("defineImagePath") == "true"));

            }
            else if(xml.name() == "warning"){
                if(xml.attributes().hasAttribute("disableAll"))
                    ret->disableAllWarnings = ((xml.attributes().value("disableAll") == "true"));
            }
            else if(xml.name() == "disabledWarnings"){
                if(xml.attributes().hasAttribute("message"))
                    ret->disabledWarningMessages.append(xml.attributes().value("message").toString());
            }
        }
        readedToken = xml.readNext();
    }
    f.close();
    mainWnd::getAppConfig()->registerRecentPrj(fileName);
    mainWnd::getMainWnd()->updateRecentPrj();
    ret->update(true);
    ret->setMainDlg(mainDlgName);
    return ret;
}

int storageManager::getProjectVersion(QString fileName)
{
  int ret = 0;
  QFile f(fileName);
  QXmlStreamReader xml;
  f.open(QIODevice::ReadOnly);
  xml.setDevice(&f);
  QXmlStreamReader::TokenType readedToken;

  readedToken = xml.readNext();
  while(!xml.atEnd())
    {
      if(readedToken == QXmlStreamReader::StartElement)
        {
          if(xml.name() == "project")
            {
              if(xml.attributes().hasAttribute("fileVersion"))
                ret = xml.attributes().value("fileVersion").toInt();
              else
                ret = 1;

            }
        }
      readedToken = xml.readNext();
    }
  f.close();
  return ret;
}
